/*******************************************************************************
*
*  Motorola Inc.
*  (c) Copyright 2003 Motorola Inc.
*  ALL RIGHTS RESERVED.
*
********************************************************************************
*
*
*  File Name:   itcn.h
*  
*  Description: Header file for Interrupt Controller (ITCN) driver
*
*
*  Modules Included:
*
*  Notes: 
*
*
*******************************************************************************/
#ifndef __ITCN_H
#define __ITCN_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before intc.h
#endif

/* appconfig.h must be included above this header file. */
#ifndef __APPCONFIG_H
	#error appconfig.h must be included above itcn.h or ITCN commands will not work correctly!
#endif

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************
	ITCN module identifier, correspond to module base address
*******************************************************************************/
#define ITCN		(&ArchIO.IntController)



/******************************************************************************
*
*                      General Interface Description
*
* The IPbus interrupt controller (ITCN) accepts interrupt request (IRQ) signals
* for interrupt priority (IP) bus-based peripherals, assigns user-defined levels
* to each interrupt requests (IRQ), and then selects the highest pending IRQ for
* presentation to the DSP56800 core. The DSP56800 core includes circuitry supports
* with seven levels of interrupts. The DSP56800 provides priority to the interrupts
* associated with the highest level. The ITCN augments the DSP56800 interrupt
* controller by expanding the maximum number of peripheral interrupts to 64 and
* adding the ability to assign each of the 64 interrupt request sources to one of
* seven levels. Within any one of the seven levels, any number of interrupt sources
* may be assigned.
* 
* For more information, please refer to the DSP56F80x User's Manual,
* for a description of the ITCN device.
*
* The ITCN device driver interface uses "ioctl" call to control specific
* ITCN functions during operation. For details see "ioctl" call.
*      
*
******************************************************************************/

/*******************************************************************************
	Configurable items, i.e. defines, for appconfig.h
********************************************************************************
For peripherals:
	#define INT_PRIORITY_LEVEL_xx 0-7		(0 means interrupt is disabled)
	(xx is number of interrupt vector and can be 10,11......62,63)

For external interrupts:
IRQA:
	void IRQAisr(void);
	#define INT_VECTOR_ADDR_8	IRQAisr
	#define IRQA_ENABLED 		0 (disabled) or 1 (enabled)
	#define IRQA_TRIGGER		0 (low level sensitive), 1 (high level sensitive),
								4 (falling edge sensitive), 5 (rising edge sensitive)

IRQB:
	void IRQBisr(void);
	#define INT_VECTOR_ADDR_9	IRQBisr
	#define IRQB_ENABLED		0 (disabled) or 1 (enabled)
	#define IRQB_TRIGGER		0 (low level sensitive), 1 (high level sensitive),
								4 (falling edge sensitive), 5 (rising edge sensitive)
*/



/*******************************************************************************
	ioctl() commands
********************************************************************************
command								parameters and commentary */
#define ITCN_INIT_GPRS				/* NULL, set all group priority registers (GPRyy) according to INT_PRIORITY_LEVEL_xx defined in appconfig.h */
#define ITCN_INIT_IPR				/* NULL, enable interrupt channels and configure external interrupts
									(all in IPR) according to INT_PRIORITY_LEVEL_xx and 
									IRQA_TRIGGER, IRQB_TRIGGER, IRQA_ENABLED, IRQB_ENABLED defined in appconfig.h */
#define ITCN_WRITE_IPR				/* 16 bit value, write value to Interrupt Priority Register (IPR) */
#define ITCN_READ_IPR				/* NULL, returns UWord16 */
#define ITCN_ENABLE_INT_CHANNEL		/* ITCN_CHANNEL0 | ITCN_CHANNEL1 | ITCN_CHANNEL2 | ITCN_CHANNEL3 | ITCN_CHANNEL4 | ITCN_CHANNEL5 | ITCN_CHANNEL6 */
#define ITCN_DISABLE_INT_CHANNEL	/* ITCN_CHANNEL0 | ITCN_CHANNEL1 | ITCN_CHANNEL2 | ITCN_CHANNEL3 | ITCN_CHANNEL4 | ITCN_CHANNEL5 | ITCN_CHANNEL6 */

//#define ITCN_ENABLE_IRQ			/* ITCN_IRQA | ITCN_IRQB */
//#define ITCN_DISABLE_IRQ			/* ITCN_IRQA | ITCN_IRQB */

#define ITCN_INT_ENABLE				/* ITCN_IRQA | ITCN_IRQB */
#define ITCN_INT_DISABLE			/* ITCN_IRQA | ITCN_IRQB */

#define ITCN_IRQA_TRIGGER_MODE		/* ITCN_LOW_LEVEL_SENSITIVE, ITCN_HIGH_LEVEL_SENSITIVE, ITCN_FALLING_EDGE_SENSITIVE, ITCN_RISING_EDGE_SENSITIVE */
#define ITCN_IRQB_TRIGGER_MODE		/* ITCN_LOW_LEVEL_SENSITIVE, ITCN_HIGH_LEVEL_SENSITIVE, ITCN_FALLING_EDGE_SENSITIVE, ITCN_RISING_EDGE_SENSITIVE */
#define ITCN_SET_INT_PRIORITY_10	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_11	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_12	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_13	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_14	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_15	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_16	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_17	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_18	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_19	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_20	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_21	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_22	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_23	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_24	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_25	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_26	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_27	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_28	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_29	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_30	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_31	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_32	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_33	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_34	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_35	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_36	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_37	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_38	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_39	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_40	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_41	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_42	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_43	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_44	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_45	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_46	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_47	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_48	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_49	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_50	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_51	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_52	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_53	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_54	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_55	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_56	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_57	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_58	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_59	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_60	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_61	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_62	/* value 0 to 7*/
#define ITCN_SET_INT_PRIORITY_63	/* value 0 to 7*/




/*******************************************************************************
	IPR bits
*******************************************************************************/
#define ITCN_IPR_IAINV			0x0001
#define ITCN_IPR_IAL0			0x0002
#define ITCN_IPR_IAL1			0x0004
#define ITCN_IPR_IBINV			0x0008
#define ITCN_IPR_IBL0			0x0010
#define ITCN_IPR_IBL1			0x0020



/*******************************************************************************
	ITCN constants used in ioctl()
*******************************************************************************/
/* Interrupt Channels (used in ITCN_ENABLE_INT_CHANNEL and 
					   ITCN_DISABLE_INT_CHANNEL commands) */
#define ITCN_CHANNEL0				0x8000
#define ITCN_CHANNEL1				0x4000
#define ITCN_CHANNEL2				0x2000
#define ITCN_CHANNEL3				0x1000
#define ITCN_CHANNEL4				0x0800
#define ITCN_CHANNEL5				0x0400
#define ITCN_CHANNEL6				0x0200

/* External interrupts (used in ITCN_ENABLE_IRQ and ITCN_DISABLE_IRQ commands) */
#define ITCN_IRQA					ITCN_IPR_IAL0
#define ITCN_IRQB					ITCN_IPR_IBL0

/* IRQ Trigger Mode (used in ITCN_IRQx_TRIGGER_MODE commands) */
#define ITCN_LOW_LEVEL_SENSITIVE	0x0000
#define ITCN_HIGH_LEVEL_SENSITIVE	ITCN_IPR_IAINV
#define ITCN_FALLING_EDGE_SENSITIVE	ITCN_IPR_IAL1
#define ITCN_RISING_EDGE_SENSITIVE	(ITCN_IPR_IAL1 | ITCN_IPR_IAINV)



/*******************************************************************************
	Implementation
*******************************************************************************/
#define PLR_TO_IPR_MASK(x) 		((x)==0 ? 0 : 0x8000>>((x)-1))
#define PLR_TO_GPR(x0,x1,x2,x3) ((x0) + 0x10*(x1) + 0x100*(x2) + 0x1000*(x3))


#include "itcntarget.h"


/* If no definition was done define reset values */
#ifndef IRQB_TRIGGER					/* ITCN_LOW_LEVEL_SENSITIVE, ITCN_HIGH_LEVEL_SENSITIVE, ITCN_FALLING_EDGE_SENSITIVE, ITCN_RISING_EDGE_SENSITIVE */
	#define IRQB_TRIGGER 	ITCN_LOW_LEVEL_SENSITIVE
#endif

#ifndef IRQB_ENABLED	/* 0 - disabled, 1 - enabled */
	#define IRQB_ENABLED 	0
#endif

#ifndef IRQA_TRIGGER					/* ITCN_LOW_LEVEL_SENSITIVE, ITCN_HIGH_LEVEL_SENSITIVE, ITCN_FALLING_EDGE_SENSITIVE, ITCN_RISING_EDGE_SENSITIVE */
	#define IRQA_TRIGGER 	ITCN_LOW_LEVEL_SENSITIVE
#endif

#ifndef IRQA_ENABLED					/* 0 - disabled, 1 - enabled */
	#define IRQA_ENABLED 	0
#endif


#ifndef INT_PRIORITY_LEVEL_10
	#define INT_PRIORITY_LEVEL_10 0
#endif

#ifndef INT_PRIORITY_LEVEL_11
	#define INT_PRIORITY_LEVEL_11 0
#endif

#ifndef INT_PRIORITY_LEVEL_12
	#define INT_PRIORITY_LEVEL_12 0
#endif

#ifndef INT_PRIORITY_LEVEL_13
	#define INT_PRIORITY_LEVEL_13 0
#endif

#ifndef INT_PRIORITY_LEVEL_14
	#define INT_PRIORITY_LEVEL_14 0
#endif

#ifndef INT_PRIORITY_LEVEL_15
	#define INT_PRIORITY_LEVEL_15 0
#endif

#ifndef INT_PRIORITY_LEVEL_16
	#define INT_PRIORITY_LEVEL_16 0
#endif

#ifndef INT_PRIORITY_LEVEL_17
	#define INT_PRIORITY_LEVEL_17 0
#endif

#ifndef INT_PRIORITY_LEVEL_18
	#define INT_PRIORITY_LEVEL_18 0
#endif

#ifndef INT_PRIORITY_LEVEL_19
	#define INT_PRIORITY_LEVEL_19 0
#endif

#ifndef INT_PRIORITY_LEVEL_20
	#define INT_PRIORITY_LEVEL_20 0
#endif

#ifndef INT_PRIORITY_LEVEL_21
	#define INT_PRIORITY_LEVEL_21 0
#endif

#ifndef INT_PRIORITY_LEVEL_22
	#define INT_PRIORITY_LEVEL_22 0
#endif

#ifndef INT_PRIORITY_LEVEL_23
	#define INT_PRIORITY_LEVEL_23 0
#endif

#ifndef INT_PRIORITY_LEVEL_24
	#define INT_PRIORITY_LEVEL_24 0
#endif

#ifndef INT_PRIORITY_LEVEL_25
	#define INT_PRIORITY_LEVEL_25 0
#endif

#ifndef INT_PRIORITY_LEVEL_26
	#define INT_PRIORITY_LEVEL_26 0
#endif

#ifndef INT_PRIORITY_LEVEL_27
	#define INT_PRIORITY_LEVEL_27 0
#endif

#ifndef INT_PRIORITY_LEVEL_28
	#define INT_PRIORITY_LEVEL_28 0
#endif

#ifndef INT_PRIORITY_LEVEL_29
	#define INT_PRIORITY_LEVEL_29 0
#endif

#ifndef INT_PRIORITY_LEVEL_30
	#define INT_PRIORITY_LEVEL_30 0
#endif

#ifndef INT_PRIORITY_LEVEL_31
	#define INT_PRIORITY_LEVEL_31 0
#endif

#ifndef INT_PRIORITY_LEVEL_32
	#define INT_PRIORITY_LEVEL_32 0
#endif

#ifndef INT_PRIORITY_LEVEL_33
	#define INT_PRIORITY_LEVEL_33 0
#endif

#ifndef INT_PRIORITY_LEVEL_34
	#define INT_PRIORITY_LEVEL_34 0
#endif

#ifndef INT_PRIORITY_LEVEL_35
	#define INT_PRIORITY_LEVEL_35 0
#endif

#ifndef INT_PRIORITY_LEVEL_36
	#define INT_PRIORITY_LEVEL_36 0
#endif

#ifndef INT_PRIORITY_LEVEL_37
	#define INT_PRIORITY_LEVEL_37 0
#endif

#ifndef INT_PRIORITY_LEVEL_38
	#define INT_PRIORITY_LEVEL_38 0
#endif

#ifndef INT_PRIORITY_LEVEL_39
	#define INT_PRIORITY_LEVEL_39 0
#endif

#ifndef INT_PRIORITY_LEVEL_40
	#define INT_PRIORITY_LEVEL_40 0
#endif

#ifndef INT_PRIORITY_LEVEL_41
	#define INT_PRIORITY_LEVEL_41 0
#endif

#ifndef INT_PRIORITY_LEVEL_42
	#define INT_PRIORITY_LEVEL_42 0
#endif

#ifndef INT_PRIORITY_LEVEL_43
	#define INT_PRIORITY_LEVEL_43 0
#endif

#ifndef INT_PRIORITY_LEVEL_44
	#define INT_PRIORITY_LEVEL_44 0
#endif

#ifndef INT_PRIORITY_LEVEL_45
	#define INT_PRIORITY_LEVEL_45 0
#endif

#ifndef INT_PRIORITY_LEVEL_46
	#define INT_PRIORITY_LEVEL_46 0
#endif

#ifndef INT_PRIORITY_LEVEL_47
	#define INT_PRIORITY_LEVEL_47 0
#endif

#ifndef INT_PRIORITY_LEVEL_48
	#define INT_PRIORITY_LEVEL_48 0
#endif

#ifndef INT_PRIORITY_LEVEL_49
	#define INT_PRIORITY_LEVEL_49 0
#endif

#ifndef INT_PRIORITY_LEVEL_50
	#define INT_PRIORITY_LEVEL_50 0
#endif

#ifndef INT_PRIORITY_LEVEL_51
	#define INT_PRIORITY_LEVEL_51 0
#endif

#ifndef INT_PRIORITY_LEVEL_52
	#define INT_PRIORITY_LEVEL_52 0
#endif

#ifndef INT_PRIORITY_LEVEL_53
	#define INT_PRIORITY_LEVEL_53 0
#endif

#ifndef INT_PRIORITY_LEVEL_54
	#define INT_PRIORITY_LEVEL_54 0
#endif

#ifndef INT_PRIORITY_LEVEL_55
	#define INT_PRIORITY_LEVEL_55 0
#endif

#ifndef INT_PRIORITY_LEVEL_56
	#define INT_PRIORITY_LEVEL_56 0
#endif

#ifndef INT_PRIORITY_LEVEL_57
	#define INT_PRIORITY_LEVEL_57 0
#endif

#ifndef INT_PRIORITY_LEVEL_58
	#define INT_PRIORITY_LEVEL_58 0
#endif

#ifndef INT_PRIORITY_LEVEL_59
	#define INT_PRIORITY_LEVEL_59 0
#endif

#ifndef INT_PRIORITY_LEVEL_60
	#define INT_PRIORITY_LEVEL_60 0
#endif

#ifndef INT_PRIORITY_LEVEL_61
	#define INT_PRIORITY_LEVEL_61 0
#endif

#ifndef INT_PRIORITY_LEVEL_62
	#define INT_PRIORITY_LEVEL_62 0
#endif

#ifndef INT_PRIORITY_LEVEL_63
	#define INT_PRIORITY_LEVEL_63 0
#endif



#define IPR_MASK	(PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_10) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_11) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_12) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_13) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_14) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_15) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_16) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_17) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_18) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_19) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_20) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_21) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_22) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_23) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_24) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_25) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_26) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_27) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_28) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_29) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_30) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_31) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_32) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_33) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_34) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_35) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_36) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_37) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_38) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_39) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_40) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_41) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_42) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_43) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_44) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_45) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_46) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_47) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_48) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_49) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_50) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_51) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_52) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_53) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_54) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_55) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_56) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_57) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_58) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_59) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_60) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_61) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_62) | \
					 PLR_TO_IPR_MASK(INT_PRIORITY_LEVEL_63) | \
					 (IRQB_TRIGGER<<3) | 0x0010*IRQB_ENABLED | \
					 IRQA_TRIGGER | 0x0002*IRQA_ENABLED \
					)
/* End of IPR mask definition */


#if (INT_PRIORITY_LEVEL_10!=0)||(INT_PRIORITY_LEVEL_11!=0)
	#define GPR10_TO_11 periphMemWrite(PLR_TO_GPR(0,0,INT_PRIORITY_LEVEL_10,INT_PRIORITY_LEVEL_11), &ArchIO.IntController.gpr[2])
#else
	#define GPR10_TO_11
#endif
#if (INT_PRIORITY_LEVEL_12!=0)||(INT_PRIORITY_LEVEL_13!=0)||(INT_PRIORITY_LEVEL_14!=0)||(INT_PRIORITY_LEVEL_15!=0)
	#define GPR12_TO_15 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_12,INT_PRIORITY_LEVEL_13,INT_PRIORITY_LEVEL_14,INT_PRIORITY_LEVEL_15), &ArchIO.IntController.gpr[3])
#else
	#define GPR12_TO_15
#endif
#if (INT_PRIORITY_LEVEL_16!=0)||(INT_PRIORITY_LEVEL_17!=0)||(INT_PRIORITY_LEVEL_18!=0)||(INT_PRIORITY_LEVEL_19!=0)
	#define GPR16_TO_19 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_16,INT_PRIORITY_LEVEL_17,INT_PRIORITY_LEVEL_18,INT_PRIORITY_LEVEL_19), &ArchIO.IntController.gpr[4])
#else
	#define GPR16_TO_19
#endif
#if (INT_PRIORITY_LEVEL_20!=0)||(INT_PRIORITY_LEVEL_21!=0)||(INT_PRIORITY_LEVEL_22!=0)||(INT_PRIORITY_LEVEL_23!=0)
	#define GPR20_TO_23 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_20,INT_PRIORITY_LEVEL_21,INT_PRIORITY_LEVEL_22,INT_PRIORITY_LEVEL_23), &ArchIO.IntController.gpr[5])
#else
	#define GPR20_TO_23
#endif
#if (INT_PRIORITY_LEVEL_24!=0)||(INT_PRIORITY_LEVEL_25!=0)||(INT_PRIORITY_LEVEL_26!=0)||(INT_PRIORITY_LEVEL_27!=0)
	#define GPR24_TO_27 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_24,INT_PRIORITY_LEVEL_25,INT_PRIORITY_LEVEL_26,INT_PRIORITY_LEVEL_27), &ArchIO.IntController.gpr[6])
#else
	#define GPR24_TO_27
#endif
#if (INT_PRIORITY_LEVEL_28!=0)||(INT_PRIORITY_LEVEL_29!=0)||(INT_PRIORITY_LEVEL_30!=0)||(INT_PRIORITY_LEVEL_31!=0)
	#define GPR28_TO_31 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_28,INT_PRIORITY_LEVEL_29,INT_PRIORITY_LEVEL_30,INT_PRIORITY_LEVEL_31), &ArchIO.IntController.gpr[7])
#else
	#define GPR28_TO_31
#endif
#if (INT_PRIORITY_LEVEL_32!=0)||(INT_PRIORITY_LEVEL_33!=0)||(INT_PRIORITY_LEVEL_34!=0)||(INT_PRIORITY_LEVEL_35!=0)
	#define GPR32_TO_35 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_32,INT_PRIORITY_LEVEL_33,INT_PRIORITY_LEVEL_34,INT_PRIORITY_LEVEL_35), &ArchIO.IntController.gpr[8])
#else
	#define GPR32_TO_35
#endif
#if (INT_PRIORITY_LEVEL_36!=0)||(INT_PRIORITY_LEVEL_37!=0)||(INT_PRIORITY_LEVEL_38!=0)||(INT_PRIORITY_LEVEL_39!=0)
	#define GPR36_TO_39 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_36,INT_PRIORITY_LEVEL_37,INT_PRIORITY_LEVEL_38,INT_PRIORITY_LEVEL_39), &ArchIO.IntController.gpr[9])
#else
	#define GPR36_TO_39
#endif
#if (INT_PRIORITY_LEVEL_40!=0)||(INT_PRIORITY_LEVEL_41!=0)||(INT_PRIORITY_LEVEL_42!=0)||(INT_PRIORITY_LEVEL_43!=0)
	#define GPR40_TO_43 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_40,INT_PRIORITY_LEVEL_41,INT_PRIORITY_LEVEL_42,INT_PRIORITY_LEVEL_43), &ArchIO.IntController.gpr[10])
#else
	#define GPR40_TO_43
#endif
#if (INT_PRIORITY_LEVEL_44!=0)||(INT_PRIORITY_LEVEL_45!=0)||(INT_PRIORITY_LEVEL_46!=0)||(INT_PRIORITY_LEVEL_47!=0)
	#define GPR44_TO_47 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_44,INT_PRIORITY_LEVEL_45,INT_PRIORITY_LEVEL_46,INT_PRIORITY_LEVEL_47), &ArchIO.IntController.gpr[11])
#else
	#define GPR44_TO_47
#endif
#if (INT_PRIORITY_LEVEL_48!=0)||(INT_PRIORITY_LEVEL_49!=0)||(INT_PRIORITY_LEVEL_50!=0)||(INT_PRIORITY_LEVEL_51!=0)
	#define GPR48_TO_51 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_48,INT_PRIORITY_LEVEL_49,INT_PRIORITY_LEVEL_50,INT_PRIORITY_LEVEL_51), &ArchIO.IntController.gpr[12])
#else
	#define GPR48_TO_51
#endif
#if (INT_PRIORITY_LEVEL_52!=0)||(INT_PRIORITY_LEVEL_53!=0)||(INT_PRIORITY_LEVEL_54!=0)||(INT_PRIORITY_LEVEL_55!=0)
	#define GPR52_TO_55 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_52,INT_PRIORITY_LEVEL_53,INT_PRIORITY_LEVEL_54,INT_PRIORITY_LEVEL_55), &ArchIO.IntController.gpr[13])
#else
	#define GPR52_TO_55
#endif
#if (INT_PRIORITY_LEVEL_56!=0)||(INT_PRIORITY_LEVEL_57!=0)||(INT_PRIORITY_LEVEL_58!=0)||(INT_PRIORITY_LEVEL_59!=0)
	#define GPR56_TO_59 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_56,INT_PRIORITY_LEVEL_57,INT_PRIORITY_LEVEL_58,INT_PRIORITY_LEVEL_59), &ArchIO.IntController.gpr[14])
#else
	#define GPR56_TO_59
#endif
#if (INT_PRIORITY_LEVEL_60!=0)||(INT_PRIORITY_LEVEL_61!=0)||(INT_PRIORITY_LEVEL_62!=0)||(INT_PRIORITY_LEVEL_63!=0)
	#define GPR60_TO_63 periphMemWrite(PLR_TO_GPR(INT_PRIORITY_LEVEL_60,INT_PRIORITY_LEVEL_61,INT_PRIORITY_LEVEL_62,INT_PRIORITY_LEVEL_63), &ArchIO.IntController.gpr[15])
#else
	#define GPR60_TO_63
#endif




#define ioctlITCN_INIT_GPRS(pItcnBase, param) { \
	GPR10_TO_11; \
	GPR12_TO_15; GPR16_TO_19; GPR20_TO_23; GPR24_TO_27; GPR28_TO_31; GPR32_TO_35; \
	GPR36_TO_39; GPR40_TO_43; GPR44_TO_47; GPR48_TO_51; GPR52_TO_55; GPR56_TO_59; \
	GPR60_TO_63; }

#define ioctlITCN_INIT_IPR(pItcnBase, param) \
	periphMemWrite(IPR_MASK, &ArchCore.InterruptPriorityReg)

#define ioctlITCN_WRITE_IPR(pItcnBase, param) \
	periphMemWrite(param, &ArchCore.InterruptPriorityReg)

#define ioctlITCN_READ_IPR(pItcnBase, param) \
	periphMemRead(&ArchCore.InterruptPriorityReg)



#define ioctlITCN_SET_INT_PRIORITY_10(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[2]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[2]); }

#define ioctlITCN_SET_INT_PRIORITY_11(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[2]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[2]); }

#define ioctlITCN_SET_INT_PRIORITY_12(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[3]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[3]); }

#define ioctlITCN_SET_INT_PRIORITY_13(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[3]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[3]); }

#define ioctlITCN_SET_INT_PRIORITY_14(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[3]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[3]); }

#define ioctlITCN_SET_INT_PRIORITY_15(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[3]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[3]); }

#define ioctlITCN_SET_INT_PRIORITY_16(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[4]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[4]); }

#define ioctlITCN_SET_INT_PRIORITY_17(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[4]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[4]); }

#define ioctlITCN_SET_INT_PRIORITY_18(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[4]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[4]); }

#define ioctlITCN_SET_INT_PRIORITY_19(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[4]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[4]); }

#define ioctlITCN_SET_INT_PRIORITY_20(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[5]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[5]); }

#define ioctlITCN_SET_INT_PRIORITY_21(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[5]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[5]); }

#define ioctlITCN_SET_INT_PRIORITY_22(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[5]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[5]); }

#define ioctlITCN_SET_INT_PRIORITY_23(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[5]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[5]); }

#define ioctlITCN_SET_INT_PRIORITY_24(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[6]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[6]); }

#define ioctlITCN_SET_INT_PRIORITY_25(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[6]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[6]); }

#define ioctlITCN_SET_INT_PRIORITY_26(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[6]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[6]); }

#define ioctlITCN_SET_INT_PRIORITY_27(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[6]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[6]); }

#define ioctlITCN_SET_INT_PRIORITY_28(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[7]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[7]); }

#define ioctlITCN_SET_INT_PRIORITY_29(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[7]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[7]); }

#define ioctlITCN_SET_INT_PRIORITY_30(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[7]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[7]); }

#define ioctlITCN_SET_INT_PRIORITY_31(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[7]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[7]); }

#define ioctlITCN_SET_INT_PRIORITY_32(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[8]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[8]); }

#define ioctlITCN_SET_INT_PRIORITY_33(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[8]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[8]); }

#define ioctlITCN_SET_INT_PRIORITY_34(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[8]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[8]); }

#define ioctlITCN_SET_INT_PRIORITY_35(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[8]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[8]); }

#define ioctlITCN_SET_INT_PRIORITY_36(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[9]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[9]); }

#define ioctlITCN_SET_INT_PRIORITY_37(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[9]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[9]); }

#define ioctlITCN_SET_INT_PRIORITY_38(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[9]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[9]); }

#define ioctlITCN_SET_INT_PRIORITY_39(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[9]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[9]); }

#define ioctlITCN_SET_INT_PRIORITY_40(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[10]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[10]); }

#define ioctlITCN_SET_INT_PRIORITY_41(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[10]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[10]); }

#define ioctlITCN_SET_INT_PRIORITY_42(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[10]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[10]); }

#define ioctlITCN_SET_INT_PRIORITY_43(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[10]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[10]); }

#define ioctlITCN_SET_INT_PRIORITY_44(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[11]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[11]); }

#define ioctlITCN_SET_INT_PRIORITY_45(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[11]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[11]); }

#define ioctlITCN_SET_INT_PRIORITY_46(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[11]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[11]); }

#define ioctlITCN_SET_INT_PRIORITY_47(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[11]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[11]); }

#define ioctlITCN_SET_INT_PRIORITY_48(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[12]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[12]); }

#define ioctlITCN_SET_INT_PRIORITY_49(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[12]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[12]); }

#define ioctlITCN_SET_INT_PRIORITY_50(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[12]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[12]); }

#define ioctlITCN_SET_INT_PRIORITY_51(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[12]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[12]); }

#define ioctlITCN_SET_INT_PRIORITY_52(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[13]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[13]); }

#define ioctlITCN_SET_INT_PRIORITY_53(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[13]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[13]); }

#define ioctlITCN_SET_INT_PRIORITY_54(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[13]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[13]); }

#define ioctlITCN_SET_INT_PRIORITY_55(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[13]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[13]); }

#define ioctlITCN_SET_INT_PRIORITY_56(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[14]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[14]); }

#define ioctlITCN_SET_INT_PRIORITY_57(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[14]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[14]); }

#define ioctlITCN_SET_INT_PRIORITY_58(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[14]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[14]); }

#define ioctlITCN_SET_INT_PRIORITY_59(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[14]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[14]); }

#define ioctlITCN_SET_INT_PRIORITY_60(pItcnBase, param) { \
	periphBitSet(param, &ArchIO.IntController.gpr[15]); \
	periphBitClear((~(param))&0x0007, &ArchIO.IntController.gpr[15]); }

#define ioctlITCN_SET_INT_PRIORITY_61(pItcnBase, param) { \
	periphBitSet((param)<<4, &ArchIO.IntController.gpr[15]); \
	periphBitClear((~((param)<<4))&0x0070, &ArchIO.IntController.gpr[15]); }

#define ioctlITCN_SET_INT_PRIORITY_62(pItcnBase, param) { \
	periphBitSet((param)<<8, &ArchIO.IntController.gpr[15]); \
	periphBitClear((~((param)<<8))&0x0700, &ArchIO.IntController.gpr[15]); }

#define ioctlITCN_SET_INT_PRIORITY_63(pItcnBase, param) { \
	periphBitSet((param)<<12, &ArchIO.IntController.gpr[15]); \
	periphBitClear((~((param)<<12))&0x7000, &ArchIO.IntController.gpr[15]); }



#define ioctlITCN_ENABLE_INT_CHANNEL(pItcnBase, param) \
	periphBitSet(param, &ArchCore.InterruptPriorityReg)

#define ioctlITCN_DISABLE_INT_CHANNEL(pItcnBase, param) \
	periphBitClear(param, &ArchCore.InterruptPriorityReg)

#define ioctlITCN_ENABLE_IRQ(pItcnBase, param) \
	periphBitSet((param), &ArchCore.InterruptPriorityReg)

#define ioctlITCN_DISABLE_IRQ(pItcnBase, param) \
	periphBitClear(param, &ArchCore.InterruptPriorityReg)

#define ioctlITCN_INT_ENABLE(pItcnBase, param) \
	periphBitSet((param), &ArchCore.InterruptPriorityReg)

#define ioctlITCN_INT_DISABLE(pItcnBase, param) \
	periphBitClear(param, &ArchCore.InterruptPriorityReg)

#define ioctlITCN_IRQA_TRIGGER_MODE(pItcnBase, param) { \
	periphBitSet((param & (ITCN_IPR_IAINV | ITCN_IPR_IAL1)), &ArchCore.InterruptPriorityReg); \
	periphBitClear(((~param) & (ITCN_IPR_IAINV | ITCN_IPR_IAL1)), &ArchCore.InterruptPriorityReg); }

#define ioctlITCN_IRQB_TRIGGER_MODE(pItcnBase, param) { \
	periphBitSet((((param)<<3) & (ITCN_IPR_IBINV | ITCN_IPR_IBL1)), &ArchCore.InterruptPriorityReg); \
	periphBitClear(((~((param)<<3)) & (ITCN_IPR_IBINV | ITCN_IPR_IBL1)), &ArchCore.InterruptPriorityReg); }


#ifdef __cplusplus
}
#endif

#endif