/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: periph.h
*
* Description: definition of ioctl macro,
*			   macros for memory access
*
* Modules Included:
*
*****************************************************************************/
#ifndef __PERIPH_H
#define __PERIPH_H

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************
* ioctl commands macro 
*******************************************************/
#define ioctl(fd,cmd,pParams) ioctl ## cmd( ## fd,pParams)


/*******************************************************
* read / write commands macro 
*******************************************************/
#define write(fd,mode,address,size) write_##fd##_##mode(address,size)
#define read(fd,mode,address,size)  read_##fd##_##mode(address,size)


/*******************************************************
* Routines for Peripheral Memory Access
********************************************************
*  Parameter usage:
*   Addr - architecture structure field 
*   Mask - bit to manipulate 
*   Data - 16 bit word to assign or access  
********************************************************/


/* void periphBitSet(UWord16 Mask, volatile UWord16 * Addr); */
#define periphBitSet(Mask, Addr)        *(Addr) |= Mask
                                        /* asm(bfset Mask,Addr) */

/* void periphBitClear(UWord16 Mask, volatile UWord16 * Addr); */
#define periphBitClear(Mask, Addr)      *(Addr) &= ~(Mask)
                                         /* asm(bfclr    Mask,Addr) */
/* set group of bits using an accumulator */
#define periphBitGrpSet(GroupMask, Mask, Addr) *(Addr) = (*(Addr) & ~(GroupMask)) | (Mask)

/* set group of bits using two non-interruptible instructions (set,reset) */
#define periphBitGrpSR(GroupMask, Mask, Addr) \
	{*(Addr) |= ((Mask) & (GroupMask)); /* set "one" bits */	\
	 *(Addr) &= ~((~(Mask)) & (GroupMask));} /* clear "zero" bits */

/* set group of bits using two non-interruptible instructions (reset,set) */
#define periphBitGrpRS(GroupMask, Mask, Addr) \
	{*(Addr) &= ~((~(Mask)) & (GroupMask)); /* clear "zero" bits */	\
	 *(Addr) |= ((Mask) & (GroupMask));} /* set "one" bits */

/* set group of bits using two non-interruptible instructions (zero,set) */
#define periphBitGrpZS(GroupMask, Mask, Addr) \
	{*(Addr) &= ~(GroupMask); /* clear all bits in group */	\
	 *(Addr) |= ((Mask) & (GroupMask));} /* set "one" bits */

/* clear (acknowlege) bit flags which are active-high and are cleared-by-write-one */

#define periphSafeAckByOne(allAckBitsGrp, bitsToAck, addr) \
	if((~(bitsToAck)) & (allAckBitsGrp))	\
		periphBitClear((~(bitsToAck)) & (allAckBitsGrp), addr);	\
	else	\
		periphMemDummyReadWrite(addr)


/* void periphBitChange(UWord16 Mask, volatile UWord16 * Addr); */
#define periphBitChange(Mask, Addr)     ( *(Addr) ^= Mask )
                                        /* asm(bfchg    Mask,Addr) */

/* bool periphBitTest(UWord16 Mask, volatile UWord16 * Addr); */
#define periphBitTest(Mask, Addr)       ( *(Addr) & (Mask) )

/* void periphMemInvBitSet(UWord16 Mask, volatile UWord16 * Addr); */
#define periphMemInvBitSet(Mask, Addr)   (*Addr = Mask | ~(*Addr))

/* void periphBitWordSet(UWord16 Mask, volatile UWord16 * Addr); */
#define periphBitWordSet(Mask, Addr)   (*Addr = Mask | *Addr)

/* void periphBitWordClear(UWord16 Mask, volatile UWord16 * Addr); */
#define periphBitWordClear(Mask, Addr) (*Addr = Mask & *Addr)


/* UWord16 periphMemRead(volatile UWord16 * Addr); */
#define periphMemRead(Addr) ((UWord16)(*Addr))

/* void periphMemWrite(UWord16 Data, volatile UWord16 * Addr); */
#define periphMemWrite(Data, Addr) ( (*(Addr)) = Data )

/* a direct address is read and then overwritten with the value read */
static asm void periphMemDummyReadWrite(volatile UWord16* addr) 
{
 bfset #0,X:(R2+0)
 rts
}

/* a direct address is read */
static asm void periphMemDummyRead(volatile UWord16* addr) 
{
 move X:(R2+0),R2
 rts
}

#ifdef __cplusplus
}
#endif

#endif