/************************************************************************
 *                                                                      *
 * Motorola, Inc.                                                       *
 * (c) Copyright 2002 Motorola, Inc.                                    *
 * ALL RIGHTS RESERVED                                                  *
 *                                                                      *   
 ************************************************************************
 *                                                                      *
 * File name  : svm.h                                                   *
 *                                                                      *
 ************************************************************************/
 
#ifndef __SVM_H
#define __SVM_H    
/********************************************************************
 * Function calculates the appropriate duty cycles needed to        *
 * generate a given stator reference voltage using the Space Vector *
 * Modulation with a duty cycle of the null switching state combined*
 * from states O000 and O111 in each sector of the hexagon.         *
 ********************************************************************/
extern int asm MCLIB_SvmStd (mc_sPhase *p_AlphaBeta, mc_s3PhaseSystem *p_abc);
    
/********************************************************************
 * Function calculates the appropriate duty cycles needed to        *
 * generate a given stator reference voltage using the Space Vector *
 * Modulation with a duty cycle of the null switching state combined* 
 * only from states O000 in each sector of the hexagon.             *
 ********************************************************************/
extern int asm MCLIB_SvmU0n(mc_sPhase * p_AlphaBeta, mc_s3PhaseSystem *p_abc);
    
    
/********************************************************************
 * Function calculates the appropriate duty cycles needed to        *
 * generate a given stator reference voltage using the Space Vector *
 * Modulation with a duty cycle of the null switching state combined*
 * only from states O111 in each sector of the hexagon.             *
 ********************************************************************/
extern int asm MCLIB_SvmU7n (mc_sPhase * p_AlphaBeta, mc_s3PhaseSystem *p_abc);
    
/********************************************************************
 * Function calculates the appropriate duty cycles needed to        *
 * generate a given stator reference voltage using the Space Vector *
 * Modulation with a duty cycle of the null switching state combined*
 * from state O000 and O111 in the even (2,4,6) and odd (1,3,5)     *
 * sectors of the hexagon, respectively.                            *
 ********************************************************************/
extern int asm MCLIB_SvmAlt (mc_sPhase *p_AlphaBeta, mc_s3PhaseSystem *p_abc);
                           
/********************************************************************   
 * Function calculates the appropriate duty cycles needed to        *
 * generate a given stator reference voltage using the General      *
 * Sinusoidal Modulation technique.                                 *
 ********************************************************************/                          
extern int asm MCLIB_PwmIct (mc_sPhase *p_AlphaBeta, mc_s3PhaseSystem *p_abc);                            

/********************************************************************
 * Function calculates the appropriate duty cycles needed to        *
 * generate a given stator reference voltage using the General      *
 * Sinusoidal Modulation with an injection of the third harmonic.   *
 ********************************************************************/
extern int asm MCLIB_SvmSci (mc_sPhase *p_AlphaBeta, mc_s3PhaseSystem *p_abc) ;

/********************************************************************
 * Function is used for elimination of the DC-Bus voltage ripple.   *
 ********************************************************************/
extern void MCLIB_ElimDcBusRip (Frac16 invModIndex, Frac16 u_DcBusMsr, 
                        mc_sPhase *pInp_AlphaBeta, mc_sPhase *pOut_AlphaBeta);                        

#endif